<?php

declare(strict_types=1);
require __DIR__ . '/vendor/autoload.php';

use ICal\ICal;
use Symfony\Component\HttpClient\CachingHttpClient;
use Symfony\Component\HttpClient\HttpClient;
use Symfony\Component\HttpKernel\HttpCache\Store;
use Twig\TwigFilter;
use Twig\NodeVisitor\OptimizerNodeVisitor;
use DorfJetzt\State;

#region Configuration
const VISITORS_FILE = '/opt/dorf.jetzt_visitors';
const ICAL_URL = 'https://chaosdorf.de/~derf/cccd.ics';
const HTTP_CACHE = '/tmp/dorf.jetzt/http_cache';
const TMPL_CACHE = '/tmp/dorf.jetzt/twig_cache';
const ROOM_STATE_URL = 'https://chaosdorf.de/raumstatus/status.png';
const DEFAULT_TZ = 'Europe/Berlin';
#endregion


const INVALID_UAS = [
	"AhrefsBot",
	"Googlebot",
	"Yahoo",
	"Go-http-client/",
	"bingbot",
	"CheckMarkNetwork",
	"SemrushBot",
	"BingPreview",
	"facebookexternalhit",
	"hetrix.tools",
];

const HASH_TO_STATE = [
	'bff0167ed8aba031c49122ef4046cf1b' => 'closed',
	'd8ec899c69283bc775952a767db9d5f5' => 'maybe_open',
	'2c2672c641425e5b2acd6ee74f39ae60' => 'open',
	'66aece8ae27ffd3a656d42005fa3efbd' => 'private',
	'86c75c0ad413b06ff8291673162d0b64' => 'unknown',
	'0' => 'error',
];

function stateMap(string $state): State
{
	return match ($state) {
		'closed' => new State(
			'Das Dorf ist gerade <em>geschlossen</em>.',
			'lock',
			'red',
		),
		'maybe_open' => new State(
			'Das Dorf ist gerade <em>vielleicht geöffnet</em>: </p><p>Der Clubraum ist offen, aber es findet keine Veranstaltung statt.</p><p>
		Der Status kann sich also kurzfristig ändern.',
			'done',
			'brown',
		),
		'open' => new State(
			'Das Dorf ist gerade <em>geöffnet</em>.</p><p>
		Komm gerne vorbei.',
			'done',
			'green',
		),
		'private' => new State(
			'Das Dorf ist gerade <em>privat</em>: </p><p>Es sind Leute da, aber der Clubraum ist nicht geöffnet.</p><p>
		Komm gerne vorbei (aber frag lieber vorher, wie lange noch Leute da sind).',
			'lock',
			'fdd835',
		),
		'unknown' => new State(
			'Der Status vom Dorf ist gerade <em>unbekannt</em>',
			'warning',
			'orange',
		),
		'error' => new State(
			'Der Server konnte den Status vom Dorf nicht abrufen.',
			'error',
			'blue',
		),
	};
}

function hasValidUa(): bool
{
	if (isset($_SERVER['HTTP_USER_AGENT'])) {
		if (in_array(true, array_map(fn ($ua) => str_contains($_SERVER['HTTP_USER_AGENT'], $ua), INVALID_UAS))) {
			return false;
		}
		return true;
	}
	return false;
}

function dateTimeFromEvent(ICal $ical, object $event): DateTimeImmutable
{
	return DateTimeImmutable::createFromMutable($ical->iCalDateToDateTime($event->dtstart_array[3]));
}

/**
 * @return array{summary: string, url: string, start: DateTimeImmutable, end: DateTimeImmutable}[]
 */
function prepareEvents(ICal $ical, array $events): array
{
	$returns = [];
	foreach ($events as $event) {
		$start = dateTimeFromEvent($ical, $event);
		$end = $start->add(new DateInterval($event->duration));
		$url = strval($event->url);
		$summary = strval($event->summary);
		$returns[] = [
			'summary' => $summary,
			'url' => $url,
			'start' => $start,
			'end' => $end,
		];
	}
	return $returns;
}

function getState(\Symfony\Contracts\HttpClient\HttpClientInterface $http): State
{
	try {
		$response = $http->request('GET', ROOM_STATE_URL);
		$hash = md5($response->getContent());
		if (!array_key_exists($hash, HASH_TO_STATE)){
			error_log("Encountered unknown state hash $hash");
		}
		return stateMap(HASH_TO_STATE[$hash]);
	} catch (\Exception $e) {
		return stateMap('error');
	}
}


$store = new Store(HTTP_CACHE);
$client = HttpClient::create();
$client = new CachingHttpClient($client, $store, ['default_ttl' => 60, 'allow_revalidate' => true]);

$state_obj = getState($client);



$ical = new ICal(options: [
	'defaultSpan' => 2,
	'defaultTimeZone' => DEFAULT_TZ,
	'defaultWeekStart' => 'MO',
	'filterDaysBefore' => '1',
]);
$ical->initUrl(ICAL_URL, userAgent: 'dorf.jetzt', acceptLanguage: 'de');
$events = $ical->eventsFromInterval('2 week') or [];
if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
	$locale = locale_accept_from_http($_SERVER['HTTP_ACCEPT_LANGUAGE']);
	setlocale(LC_TIME, $locale);
}
$visitors = file_get_contents(VISITORS_FILE);
if (is_string($visitors)) {
	$visitors = intval($visitors);
} else {
	$visitors = 0;
}



$loader = new \Twig\Loader\FilesystemLoader('templates');
$twig = new \Twig\Environment($loader, [
	'cache' => TMPL_CACHE,
	'auto_reload' => true,
	// The 'raw' optimizer sometimes eats the only 'raw' that is used
	'optimizations' => OptimizerNodeVisitor::OPTIMIZE_ALL ^ OptimizerNodeVisitor::OPTIMIZE_RAW_FILTER,
]);

function formatEndDt(?DateTimeImmutable $end, ?DateTimeImmutable $start, string $timezone = 'Europe/Berlin'): string
{
	if ($end == null || $start == null) return "ERROR";
	$tz = new DateTimeZone($timezone);
	$daySame = $end->setTimeZone($tz)->format('d.m.Y') == $start->setTimeZone($tz)->format('d.m.Y');
	$endIsMidnight = $end->setTimeZone($tz)->format('H:i') == '00:00' && $start->setTimeZone($tz)->format('H:i') != '00:00';
	if ($daySame || $endIsMidnight) {
		return $end->format('H:i');
	} else {
		return $end->format('d.m.Y H:i');
	}
}

$twig->addFilter(new TwigFilter('end_datetime', 'formatEndDt'));
$twig->getExtension(\Twig\Extension\CoreExtension::class)->setTimezone(DEFAULT_TZ);
echo ($twig->render("Main.twig", [
	'visitors' => $visitors,
	'state_svg' => $state_obj->svg_name,
	'state_color' => $state_obj->color,
	'state_string' => $state_obj->description,
	'events' => prepareEvents($ical, $events),
]));
/* Initialising values */
if (hasValidUa()) {
	$visitors++;
	file_put_contents(VISITORS_FILE, strval($visitors));
}
